# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'PodcastChannel.slug'
        db.alter_column(u'podcast_client_podcastchannel', 'slug', self.gf('django_extensions.db.fields.AutoSlugField')(allow_duplicates=False, max_length=50, separator=u'-', populate_from='title', overwrite=True))
        # Adding field 'PodcastItem.slug'
        db.add_column(u'podcast_client_podcastitem', 'slug',
                      self.gf('django_extensions.db.fields.AutoSlugField')(allow_duplicates=False, max_length=50, separator=u'-', blank=True, default='', populate_from=('channel', 'title'), overwrite=False),
                      keep_default=False)

        # Adding field 'PodcastItem.link'
        db.add_column(u'podcast_client_podcastitem', 'link',
                      self.gf('django.db.models.fields.URLField')(default='', max_length=200, blank=True),
                      keep_default=False)

        # Adding field 'PodcastItem.cover_url'
        db.add_column(u'podcast_client_podcastitem', 'cover_url',
                      self.gf('django.db.models.fields.URLField')(default='', max_length=200, blank=True),
                      keep_default=False)


    def backwards(self, orm):

        # Changing field 'PodcastChannel.slug'
        db.alter_column(u'podcast_client_podcastchannel', 'slug', self.gf('django_extensions.db.fields.AutoSlugField')(populate_from='title', allow_duplicates=False, max_length=50, separator=u'-', overwrite=False))
        # Deleting field 'PodcastItem.slug'
        db.delete_column(u'podcast_client_podcastitem', 'slug')

        # Deleting field 'PodcastItem.link'
        db.delete_column(u'podcast_client_podcastitem', 'link')

        # Deleting field 'PodcastItem.cover_url'
        db.delete_column(u'podcast_client_podcastitem', 'cover_url')


    models = {
        u'podcast_client.podcastchannel': {
            'Meta': {'ordering': "('-modified', '-created')", 'object_name': 'PodcastChannel'},
            'copyright': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'cover_url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'download_new': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'slug': ('django_extensions.db.fields.AutoSlugField', [], {'allow_duplicates': 'False', 'max_length': '50', 'separator': "u'-'", 'blank': 'True', 'populate_from': "'title'", 'overwrite': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'url': ('django.db.models.fields.URLField', [], {'unique': 'True', 'max_length': '200'}),
            'website': ('django.db.models.fields.URLField', [], {'max_length': '200', 'blank': 'True'})
        },
        u'podcast_client.podcastitem': {
            'Meta': {'ordering': "('-publish_date',)", 'object_name': 'PodcastItem'},
            'author': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'channel': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'podcast_items'", 'to': u"orm['podcast_client.PodcastChannel']"}),
            'cover_url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'file_type': ('django.db.models.fields.CharField', [], {'max_length': '20', 'blank': 'True'}),
            'guid': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'link': ('django.db.models.fields.URLField', [], {'max_length': '200', 'blank': 'True'}),
            'listened': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'publish_date': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'slug': ('django_extensions.db.fields.AutoSlugField', [], {'allow_duplicates': 'False', 'max_length': '50', 'separator': "u'-'", 'blank': 'True', 'populate_from': "('channel', 'title')", 'overwrite': 'False'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'})
        }
    }

    complete_apps = ['podcast_client']


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'PodcastChannel'
        db.create_table(u'podcast_client_podcastchannel', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now, blank=True)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now, blank=True)),
            ('url', self.gf('django.db.models.fields.URLField')(unique=True, max_length=200)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255, blank=True)),
            ('slug', self.gf('django_extensions.db.fields.AutoSlugField')(allow_duplicates=False, max_length=50, separator=u'-', blank=True, populate_from='title', overwrite=False)),
            ('description', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('website', self.gf('django.db.models.fields.URLField')(max_length=200, blank=True)),
            ('copyright', self.gf('django.db.models.fields.CharField')(max_length=255, blank=True)),
            ('cover_url', self.gf('django.db.models.fields.URLField')(max_length=200, blank=True)),
            ('download_new', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'podcast_client', ['PodcastChannel'])

        # Adding model 'PodcastItem'
        db.create_table(u'podcast_client_podcastitem', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('guid', self.gf('django.db.models.fields.CharField')(max_length=255, db_index=True)),
            ('channel', self.gf('django.db.models.fields.related.ForeignKey')(related_name='podcast_items', to=orm['podcast_client.PodcastChannel'])),
            ('url', self.gf('django.db.models.fields.URLField')(max_length=200)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255, blank=True)),
            ('description', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('author', self.gf('django.db.models.fields.CharField')(max_length=255, blank=True)),
            ('publish_date', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('file_type', self.gf('django.db.models.fields.CharField')(max_length=20, blank=True)),
            ('file', self.gf('django.db.models.fields.files.FileField')(max_length=100, null=True, blank=True)),
            ('listened', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'podcast_client', ['PodcastItem'])


    def backwards(self, orm):
        # Deleting model 'PodcastChannel'
        db.delete_table(u'podcast_client_podcastchannel')

        # Deleting model 'PodcastItem'
        db.delete_table(u'podcast_client_podcastitem')


    models = {
        u'podcast_client.podcastchannel': {
            'Meta': {'ordering': "('-modified', '-created')", 'object_name': 'PodcastChannel'},
            'copyright': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'cover_url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'download_new': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'blank': 'True'}),
            'slug': ('django_extensions.db.fields.AutoSlugField', [], {'allow_duplicates': 'False', 'max_length': '50', 'separator': "u'-'", 'blank': 'True', 'populate_from': "'title'", 'overwrite': 'False'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'url': ('django.db.models.fields.URLField', [], {'unique': 'True', 'max_length': '200'}),
            'website': ('django.db.models.fields.URLField', [], {'max_length': '200', 'blank': 'True'})
        },
        u'podcast_client.podcastitem': {
            'Meta': {'object_name': 'PodcastItem'},
            'author': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'channel': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'podcast_items'", 'to': u"orm['podcast_client.PodcastChannel']"}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'file_type': ('django.db.models.fields.CharField', [], {'max_length': '20', 'blank': 'True'}),
            'guid': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'listened': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'publish_date': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'})
        }
    }

    complete_apps = ['podcast_client']


#!/usr/bin/env python

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup
import os
import sys

version = '0.2.3'

if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    sys.exit()

if sys.argv[-1] == 'tag':
    os.system("git tag -a %s -m 'version %s'" % (version, version))
    os.system("git push --tags")
    sys.exit()

setup(name='django-podcast-client',
      version=version,
      description='A client for downloading and organzing podcasts.',
      author='Jeremy Satterfield',
      author_email='jsatt22@gmail.com',
      url='http://github.com/jsatt/django-podcast-client',
      license='GNU General Public License v3 (GPLv3)',
      packages=[
          'podcast_client', 'podcast_client.api', 'podcast_client.management',
          'podcast_client.management.commands', 'podcast_client.migrations',
      ],
      include_package_data=True,
      install_requires=[
          'South',
          'Django>=1.5, <1.7',
          'django-extensions',
          'python-dateutil',
          'requests',
          'lxml',
          'celery>=3.1',
          'djangorestframework<3.0',
      ],
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Web Environment',
          'Framework :: Django',
          'Intended Audience :: End Users/Desktop',
          'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Topic :: Utilities',],
     )



from lxml import etree
from urlparse import urlparse
import logging

from django.conf import settings
from django.db import models
from django_extensions.db.models import AutoSlugField, TimeStampedModel
import dateutil.parser
import requests

logger = logging.getLogger(__name__)

settings.PODCAST_DIRECTORY = getattr(settings, 'PODCAST_DIRECTORY', 'podcasts')


class PodcastChannel(TimeStampedModel):
    url = models.URLField(unique=True)
    title = models.CharField(max_length=255, blank=True)
    slug = AutoSlugField(populate_from='title', overwrite=True)
    description = models.TextField(blank=True)
    website = models.URLField(blank=True)
    copyright = models.CharField(max_length=255, blank=True)
    cover_url = models.URLField(blank=True)
    download_new = models.BooleanField(default=False)

    def __unicode__(self):
        return u'%s' % (self.title or self.url)

    @classmethod
    def subscribe(cls, url):
        channel = cls(url=url)
        channel.update_channel()
        return channel

    def update_channel(self, download=False):
        logger.info('Updating Channel: %s' % self)
        req = requests.get(self.url)
        if req.ok:
            tree = etree.fromstring(req.content)
            channel = tree.find('channel')
            self.title = getattr(channel.find('title'), 'text', None) or ''
            self.description = getattr(
                channel.find('description'), 'text', None) or ''
            self.website = getattr(
                channel.find('link'), 'text', None or '')
            self.copyright = getattr(
                channel.find('copyright'), 'text', None) or ''
            self.cover_url = self.parse_cover_url(channel)
            self.save()
            self.update_items(channel, download=download)
        else:
            logger.error('Failed to retrieve feed. Status %s' % req.reason)

    def parse_cover_url(self, tree):
        image_url = getattr(
            tree.find('image/url'), 'text', None) or ''

        if not image_url and 'media' in tree.nsmap:
            image_url = getattr(
                tree.find('media:thumbnail', tree.nsmap),
                'attrib', {}).get('url', '')

        if not image_url and 'itunes' in tree.nsmap:
            image_url = getattr(
                tree.find('itunes:image', tree.nsmap),
                'attrib', {}).get('href', '')

        return image_url

    def update_items(self, channel, download=False):
        new_items = []
        for item in channel.findall('item'):
            guid = getattr(item.find('guid'), 'text', None) or ''
            pod_item, created = self.podcast_items.get_or_create(guid=guid)
            if created:
                pod_item.title = getattr(
                    item.find('title'), 'text', None) or ''
                pod_item.description = getattr(
                    item.find('description'), 'text', None) or ''
                pod_item.author = getattr(
                    item.find('author'), 'text', None)or ''
                pod_item.link = getattr(item.find('link'), 'text', None) or ''
                pub_date = getattr(item.find('pubDate'), 'text', None) or ''
                if pub_date:
                    pod_item.publish_date = dateutil.parser.parse(pub_date)
                enclosure = getattr(
                    item.find('enclosure'), 'attrib', None) or ''
                if enclosure:
                    pod_item.url = enclosure.get('url', '')
                    pod_item.file_type = enclosure.get('type', '')
                pod_item.cover_url = self.parse_cover_url(item)
                pod_item.save()
                new_items.append(pod_item)
        logger.info('Found %d new items' % len(new_items))
        if self.download_new or download:
            for item in new_items:
                item.download_file()

    def has_unlistened(self):
        return self.podcast_items.filter(listened=False).exists()


class PodcastItem(models.Model):
    guid = models.CharField(max_length=255, db_index=True)
    channel = models.ForeignKey(PodcastChannel, related_name='podcast_items')
    url = models.URLField()
    title = models.CharField(max_length=255, blank=True)
    slug = AutoSlugField(populate_from=('channel', 'title'), overwrite=True)
    description = models.TextField(blank=True)
    author = models.CharField(max_length=255, blank=True)
    link = models.URLField(blank=True)
    publish_date = models.DateTimeField(blank=True, null=True)
    file_type = models.CharField(max_length=20, blank=True)
    file = models.FileField(upload_to=settings.PODCAST_DIRECTORY, blank=True,
                            null=True)
    listened = models.BooleanField(default=False)
    cover_url = models.URLField(blank=True)

    AUDIO_FORMATS = ('audio/mpeg', 'audio/mp4', 'audio/ogg', 'audio/vorbis',
                     'audio/webm', 'audio/vnd.wave')
    VIDEO_FORMATS = ('video/mpeg', 'video/mp4', 'video/ogg', 'video/webm',
                     'video/quicktime', 'video/x-flv', 'video/x-ms-wmv',
                     'video/x-m4v')

    class Meta:
        get_latest_by = 'publish_date'
        ordering = ('-publish_date',)

    def __unicode__(self):
        return u'%s - %s' % (self.channel, self.title)

    def download_file(self):
        logger.info('Downloading - %s' % self.title)
        req = requests.get(self.url, stream=True)
        if req.ok:
            # some downloads are too big to keep in memory
            filename = urlparse(self.url).path.split('/')[-1]
            file_path = '%s/%s' % (settings.PODCAST_DIRECTORY, filename)
            path = '%s/%s' % (settings.MEDIA_ROOT, file_path)
            with open(path, 'wb') as f:
                for block in req.iter_content(1024):
                    if not block:
                        break
                    f.write(block)
            self.file = file_path
            self.save()
        else:
            logger.error('Failed to retrieve file. Status %s' % req.reason)

    def delete_file(self):
        if self.file:
            self.file.delete()

    @property
    def media_type(self):
        if self.file_type in self.AUDIO_FORMATS:
            media_type = 'audio'
        elif self.file_type in self.VIDEO_FORMATS:
            media_type = 'video'
        elif self.file_type:
            media_type = 'unknown'
        else:
            media_type = 'none'
        return media_type


def cleanup_item_delete(sender, instance=None, **kwargs):
    instance.delete_file()

models.signals.pre_delete.connect(
    cleanup_item_delete, sender=PodcastItem)



from rest_framework import status
from rest_framework.generics import (
    GenericAPIView, ListCreateAPIView, RetrieveUpdateAPIView)
from rest_framework.response import Response

from . import tasks
from .api.serializers import (
    DetailPaginationSerializer, PodcastChannelDetailSerializer,
    PodcastItemDetailSerializer)
from .celery import app
from .models import PodcastChannel, PodcastItem


class ChannelDetailAPI(RetrieveUpdateAPIView):
    model = PodcastChannel
    serializer_class = PodcastChannelDetailSerializer
    lookup_field = 'slug'


class ChannelListAPI(ListCreateAPIView):
    model = PodcastChannel
    serializer_class = PodcastChannelDetailSerializer
    pagination_serializer_class = DetailPaginationSerializer
    paginate_by = 10


class ItemDetailAPI(RetrieveUpdateAPIView):
    model = PodcastItem
    serializer_class = PodcastItemDetailSerializer
    lookup_field = 'slug'


class ItemListAPI(ListCreateAPIView):
    model = PodcastItem
    serializer_class = PodcastItemDetailSerializer
    pagination_serializer_class = DetailPaginationSerializer
    paginate_by = 10

    def get_queryset(self):
        qs = super(ItemListAPI, self).get_queryset()
        return qs.filter(channel__slug=self.kwargs['slug'])


class ItemFileAPI(GenericAPIView):
    model = PodcastItem

    def delete(self, request, *args, **kwargs):
        obj = self.get_object()
        obj.delete_file()
        return Response(status=status.HTTP_204_NO_CONTENT)

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        data = {}

        if 'download_file' in request.GET:
            task = tasks.download_file.delay(self.object.id)
            data = {'status': task.status, 'task_id': task.id}
        elif 'download_status' in request.GET:
            if request.GET.get('task_id', ''):
                result = app.AsyncResult(request.GET['task_id'])
                data = {'status': result.status}
            else:
                data = {'status': 'FAILED'}

        return Response(data)



from optparse import make_option

from django.core.management.base import BaseCommand
from requests.exceptions import RequestException

from podcast_client.models import PodcastChannel


class Command(BaseCommand):
    args = '<arg1> <arg2> ...'
    help = 'Command line utility for managing podcasts in '
    'django-podcast-client'
    option_list = BaseCommand.option_list + (
        make_option('-l', '--list', action='store_true', dest='list',
                    help='List all podcasts currently subscribed'),
        make_option('-s', '--subscribe', action='store_true', dest='subscribe',
                    help='Subcribe to podcasts'),
        make_option('-u', '--update', action='store_true', dest='update',
                    help='Update podcasts'),
        make_option('-d', '--download', action='store_true', dest='download',
                    help='Download new items when updating'),
        make_option('--unsubscribe', action='store_true',
                    dest='unsubscribe',
                    help='Unsubscribe podcast and \033[4mdelete\033[0m '
                    'associated files'),
    )

    def handle(self, *args, **options):
        if options.get('subscribe'):
            self.subscribe(*args, **options)
        elif options.get('update'):
            self.update(*args, **options)
        elif options.get('list'):
            self.list(*args, **options)
        elif options.get('unsubscribe'):
            self.unsubscribe(*args, **options)

    def unsubscribe(self, *args, **options):
        try:
            channel = PodcastChannel.objects.get(slug=args[0])
        except (IndexError, PodcastChannel.DoesNotExist):
            self.stderr.write('Please enter a valid slug. Use -l or --list to '
                              'see all available slugs.')
        else:
            name = unicode(channel)
            channel.delete()
            self.stdout.write('"%s" unsubscribed and deleted.' % name)

    def list(self, *args, **options):
        for channel in PodcastChannel.objects.all():
            self.stdout.write('%s (%s)' % (channel.title, channel.slug))

    def subscribe(self, *args, **options):
        for url in args:
            try:
                PodcastChannel.subscribe(url)
            except RequestException:
                self.stderr.write('"%s" is not a valid url. Skipping.' % url)

    def update(self, *args, **options):
        if not args:
            channels = PodcastChannel.objects.all()
        else:
            channels = PodcastChannel.objects.filter(slug__in=args)

        if channels:
            for channel in channels:
               channel.update_channel(download=options['download'])
        else:
            self.stderr.write('Please enter at least one valid slug or no '
                              'slugs to update all. Use -l or --list to '
                              'see all available slugs.')
